
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: AuraButtonMixin, CreateFont, string

local BuffFrame = BuffFrame
local DebuffFrame = DebuffFrame
local next = next
local UnitAura = UnitAura
local unpack = unpack

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Buff Frame")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	self:UpdateFonts()

	if not self:IsHooked(BuffFrame, "UpdateAuraButtons") then
		self:SecureHook(BuffFrame, "UpdateAuraButtons", "BuffFrame_UpdateAuraButtons")
	end

	if not self:IsHooked(BuffFrame, "UpdateGridLayout") then
		self:SecureHook(BuffFrame, "UpdateGridLayout", "BuffFrame_UpdateAuraButtons")
	end

	if not self:IsHooked(DebuffFrame, "UpdateAuraButtons") then
		self:SecureHook(DebuffFrame, "UpdateAuraButtons", "DebuffButton_UpdateAuraButtons")
	end

	if not self:IsHooked(DebuffFrame, "UpdateGridLayout") then
		self:SecureHook(DebuffFrame, "UpdateGridLayout", "DebuffButton_UpdateAuraButtons")
	end

	if not self:IsHooked(AuraButtonMixin, "OnUpdate") then
		self:SecureHook(AuraButtonMixin, "OnUpdate", "StyleAura")
	end

	self:RegisterMessage("SIXUI_CONFIG_UPDATE")
	self:RegisterMessage("SIXUI_PROFILE_UPDATE", "UpdateAuras")
end

-- Apply SixUI's style to an aura.
function M:StyleAura(aura, filter)
	if (not aura or aura.isAuraAnchor) then return end
	local icon, count, duration, debuffBorder, tempEnchantBorder = aura.Icon, aura.Count, aura.Duration, aura.DebuffBorder, aura.TempEnchantBorder

	B:CreateBorder(aura, C.BORDER_PADDING, C.BORDER_PADDING, C.BORDER_PADDING - 10, C.BORDER_PADDING)

	if icon then
		icon:SetTexCoord(unpack(C.ZOOM_TEX_COORD))
	end

	if count then
		count:SetFontObject(self.TEXT_COUNT)
		count:SetJustifyH("RIGHT")
		count:ClearAllPoints()
		count:SetPoint("BOTTOMRIGHT", aura.Icon, "BOTTOMRIGHT", 0, 3)
	end

	if duration then
		duration:SetFontObject(self.TEXT_DURATION)
		duration:SetJustifyH("CENTER")
		duration:ClearAllPoints()
		duration:SetPoint("TOP", aura.Icon, "BOTTOM", 0, -2 + -C.BORDER_PADDING)
	end

	if debuffBorder then
		debuffBorder:Hide()
	end

	if tempEnchantBorder then
		tempEnchantBorder:Hide()
	end

	if filter == "HARMFUL" and aura.unit then
		local _, _, _, type = UnitAura(aura.unit, aura.buttonInfo.index, filter)
		local color = C.DEBUFF_TYPE_TO_COLOR[type]

		aura:GetBorder():SetColor(color.r, color.g, color.b, color.a)
	end
end

-- Update all known, shown buffs.
function M:UpdateAuras()
	self:BuffFrame_UpdateAuraButtons()
	self:DebuffButton_UpdateAuraButtons()
end

function M:UpdateFonts()
	local fontFile, fontSize = C.TEXT_REGULAR:GetFont()
	self.TEXT_COUNT = self.TEXT_COUNT or CreateFont("SixUI Buff Count")
	self.TEXT_COUNT:SetFont(fontFile, fontSize, "OUTLINE")
	self.TEXT_DURATION = self.TEXT_DURATION or CreateFont("SixUI Buff Duration")
	self.TEXT_DURATION:SetFont(fontFile, fontSize, "")
	self.TEXT_DURATION:SetShadowColor(0, 0, 0, 1)
	self.TEXT_DURATION:SetShadowOffset(1, -1)
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

-- Use a hook to anchor buffs to SixUI_BuffsAnchor.
function M:BuffFrame_UpdateAuraButtons()
	for _, aura in next, BuffFrame.auraFrames do
		self:StyleAura(aura, "HELPFUL")
	end
end

-- Use a hook to anchor debuffs to SixUI_BuffsAnchor.
function M:DebuffButton_UpdateAuraButtons(buttonName, index)
	for _, aura in next, DebuffFrame.auraFrames do
		self:StyleAura(aura, "HARMFUL")
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CONFIG_UPDATE(event, key, ...)
	local c, g, k = string.split(".", key)

	if c == "color" and g == "debuff" then
		self:UpdateAuras()
	elseif c == "font" then
		self:UpdateFonts()
	end
end
